@@ -1,7 +1,7 @@
-/*  $Id: weather.c 2366 2007-01-15 19:10:23Z nick $
+/*  $Id: weather.c 3231 2007-09-14 14:43:27Z nick $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -45,10 +45,6 @@
 
 
 
-static gint GTK_ICON_SIZE_SMALL = 0;
-
-
-
 gboolean
 check_envproxy (gchar **proxy_host,
                 gint   *proxy_port)
@@ -189,31 +185,16 @@ make_label (xml_weather *weatherdata,
 
 
 
-static gchar *
-get_filename (const xfceweather_data *data)
-{
-  gchar *filename, *fullfilename;
-
-  filename = g_strdup_printf ("xfce4/weather-plugin/weather_%s_%c.xml",
-                              data->location_code,
-                              data->unit == METRIC ? 'm' : 'i');
-
-  fullfilename = xfce_resource_save_location (XFCE_RESOURCE_CACHE, filename,
-                                              TRUE);
-  g_free (filename);
-
-  return fullfilename;
-}
-
-
-
 static void
 set_icon_error (xfceweather_data *data)
 {
-  GdkPixbuf *icon = get_icon ("99", data->iconsize);
+  GdkPixbuf   *icon;
+  gint         height;
+  gint         size;
+  gchar       *str;
+  const gchar *txtsize;
 
-  gtk_image_set_from_pixbuf (GTK_IMAGE (data->iconimage), icon);
-  g_object_unref (G_OBJECT (icon));
+  size = data->size;
 
   if (data->weatherdata)
     {
@@ -221,6 +202,31 @@ set_icon_error (xfceweather_data *data)
       data->weatherdata = NULL;
     }
 
+  /* arbitrary, choose something that works */
+  if (size > 36)
+    txtsize = "medium";
+  else if (size > 30)
+    txtsize = "small";
+  else if (size > 24)
+    txtsize = "x-small";
+  else
+    txtsize = "xx-small";
+
+  str = g_strdup_printf ("<span size=\"%s\">No Data</span>", txtsize);
+  gtk_scrollbox_set_label (GTK_SCROLLBOX (data->scrollbox), -1, str);
+  g_free (str);
+
+  gtk_scrollbox_enablecb  (GTK_SCROLLBOX (data->scrollbox), TRUE);
+
+  gtk_widget_get_size_request (data->scrollbox, NULL, &height);
+
+  icon = get_icon ("99", data->size - height - 2);
+
+  gtk_image_set_from_pixbuf (GTK_IMAGE (data->iconimage), icon);
+
+  if (G_LIKELY (icon))
+    g_object_unref (G_OBJECT (icon));
+
   gtk_tooltips_set_tip (data->tooltips, data->tooltipbox,
                         _("Cannot update weather data"), NULL);
 }
@@ -230,10 +236,11 @@ set_icon_error (xfceweather_data *data)
 static void
 set_icon_current (xfceweather_data *data)
 {
-  guint      i;
-  GdkPixbuf *icon = NULL;
-  datas      opt;
-  gchar     *str;
+  guint           i;
+  GdkPixbuf      *icon = NULL;
+  datas           opt;
+  gchar          *str;
+  gint            size, height;
 
   for (i = 0; i < data->labels->len; i++)
     {
@@ -248,9 +255,22 @@ set_icon_current (xfceweather_data *data)
 
   gtk_scrollbox_enablecb (GTK_SCROLLBOX (data->scrollbox), TRUE);
 
-  icon = get_icon (get_data (data->weatherdata, WICON), data->iconsize);
+  if (i == 0)
+    {
+      size = data->size;
+    }
+  else
+    {
+      gtk_widget_get_size_request (data->scrollbox, NULL, &height);
+      size = data->size - height - 2;
+    }
+
+  icon = get_icon (get_data (data->weatherdata, WICON), size);
+
   gtk_image_set_from_pixbuf (GTK_IMAGE (data->iconimage), icon);
-  g_object_unref (G_OBJECT (icon));
+
+  if (G_LIKELY (icon))
+    g_object_unref (G_OBJECT (icon));
 
   gtk_tooltips_set_tip (data->tooltips, data->tooltipbox,
                         translate_desc (get_data (data->weatherdata, TRANS)),
@@ -260,32 +280,31 @@ set_icon_current (xfceweather_data *data)
 
 
 static void
-cb_update (gboolean status,
-           gpointer user_data)
+cb_update (gboolean  succeed,
+           gchar    *result,
+           gpointer  user_data)
 {
-  xfceweather_data *data = (xfceweather_data *) user_data;
-  gchar            *fullfilename;
+  xfceweather_data *data = user_data;
   xmlDoc           *doc;
   xmlNode          *cur_node;
   xml_weather      *weather = NULL;
 
-  fullfilename = get_filename (data);
+  if (G_LIKELY (succeed && result))
+    {
+      doc = xmlParseMemory (result, strlen (result));
 
-  if (G_UNLIKELY (fullfilename == NULL))
-    return;
+      g_free (result);
 
-  doc = xmlParseFile (fullfilename);
-  g_free (fullfilename);
+      if (G_LIKELY (doc))
+        {
+          cur_node = xmlDocGetRootElement (doc);
 
-  if (G_UNLIKELY (doc == NULL))
-    return;
+          if (cur_node)
+            weather = parse_weather (cur_node);
 
-  cur_node = xmlDocGetRootElement (doc);
-
-  if (cur_node)
-    weather = parse_weather (cur_node);
-
-  xmlFreeDoc (doc);
+          xmlFreeDoc (doc);
+        }
+    }
 
   gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
 
@@ -300,73 +319,38 @@ cb_update (gboolean status,
   else
     {
       set_icon_error (data);
-      return;
     }
 }
 
 
 
-/* -1 error 0 no update needed 1 updating */
-static gint
-update_weatherdata (xfceweather_data *data,
-                    gboolean          force)
+static gboolean
+update_weatherdata (xfceweather_data *data)
 {
-  struct stat  attrs;
-  gchar       *fullfilename;
-  gchar       *url;
-  gboolean     status;
+  gchar *url;
 
   if (!data->location_code)
-    return -1;
-
-  fullfilename = get_filename (data);
-
-  if (!fullfilename)
-    {
-      DBG ("can't get savedir?");
-      return -1;
-    }
-
-  if (force || (stat (fullfilename, &attrs) == -1) ||
-      ((time (NULL) - attrs.st_mtime) > (UPDATE_TIME)))
-    {
-      url = g_strdup_printf ("/weather/local/%s?cc=*&dayf=%d&unit=%c",
-                             data->location_code, XML_WEATHER_DAYF_N,
-                             data->unit == METRIC ? 'm' : 'i');
-
-      status = http_get_file (url, "xoap.weather.com", fullfilename,
-                              data->proxy_host, data->proxy_port,
-                              cb_update,
-                              (gpointer) data);
-
-      g_free (url);
-      g_free (fullfilename);
-
-      return status ? 1 : -1;
-    }
-  else if (data->weatherdata)
-    {
-      return 0;
-    }
-  else
-    {
-      cb_update (TRUE, data);
-      return 1;
-    }
-}
-
-
-
-static void
-update_plugin (xfceweather_data *data,
-               gboolean          force)
-{
-  if (update_weatherdata (data, force) == -1)
     {
       gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
       set_icon_error (data);
+
+      return TRUE;
     }
-  /* else update will be called through the callback in http_get_file () */
+
+  /* build url */
+  url = g_strdup_printf ("/weather/local/%s?cc=*&dayf=%d&unit=%c",
+                         data->location_code, XML_WEATHER_DAYF_N,
+                         data->unit == METRIC ? 'm' : 'i');
+
+  /* start receive thread */
+  weather_http_receive_data ("xoap.weather.com", url, data->proxy_host,
+                             data->proxy_port, cb_update, data);
+
+  /* cleanup */
+  g_free (url);
+
+  /* keep timeout running */
+  return TRUE;
 }
 
 
@@ -387,6 +371,17 @@ labels_clear (GArray *array)
 
 
 
+static void
+xfceweather_set_visibility (xfceweather_data *data)
+{
+  if (data->labels->len > 0)
+    gtk_widget_show (data->scrollbox);
+  else
+    gtk_widget_hide (data->scrollbox);
+}
+
+
+
 static void
 xfceweather_read_config (XfcePanelPlugin  *plugin,
                          xfceweather_data *data)
@@ -522,38 +517,23 @@ xfceweather_write_config (XfcePanelPlugin  *plugin,
 
 
 
-static gboolean
-update_cb (xfceweather_data *data)
-{
-  DBG ("update_cb(): callback called");
-
-  update_plugin (data, FALSE);
-
-  DBG ("update_cb(): request added, returning");
-
-  return TRUE;
-}
-
-
-
 static void
-update_plugin_with_reset (xfceweather_data *data,
-                          gboolean          force)
+update_weatherdata_with_reset (xfceweather_data *data)
 {
   if (data->updatetimeout)
     g_source_remove (data->updatetimeout);
 
-  update_plugin (data, force);
+  update_weatherdata (data);
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_cb, data);
+    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
 }
 
 
 static void
 update_config (xfceweather_data * data)
 {
-  update_plugin_with_reset (data, TRUE);        /* force because units could have changed */
+  update_weatherdata_with_reset (data);
 }
 
 
@@ -595,7 +575,9 @@ cb_click (GtkWidget      *widget,
         }
     }
   else if (event->button == 2)
-    update_plugin_with_reset (data, TRUE);
+    {
+      update_weatherdata_with_reset (data);
+    }
 
   return FALSE;
 }
@@ -608,7 +590,7 @@ mi_click (GtkWidget *widget,
 {
   xfceweather_data *data = (xfceweather_data *) user_data;
 
-  update_plugin_with_reset (data, TRUE);
+  update_weatherdata_with_reset (data);
 }
 
 
@@ -639,6 +621,8 @@ xfceweather_dialog_response (GtkWidget          *dlg,
 
       xfce_panel_plugin_unblock_menu (data->plugin);
       xfceweather_write_config (data->plugin, data);
+
+      xfceweather_set_visibility (data);
     }
 }
 
@@ -691,9 +675,6 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
   datas             lbl;
   GdkPixbuf        *icon = NULL;
 
-  if (!GTK_ICON_SIZE_SMALL)
-    GTK_ICON_SIZE_SMALL = gtk_icon_size_register ("iconsize_small", 16, 16);
-
   data->plugin = plugin;
 
   data->tooltips = gtk_tooltips_new ();
@@ -702,10 +683,12 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
 
   data->scrollbox = gtk_scrollbox_new ();
 
-  icon = get_icon ("99", GTK_ICON_SIZE_SMALL);
+  icon = get_icon ("99", 16);
   data->iconimage = gtk_image_new_from_pixbuf (icon);
   gtk_misc_set_alignment (GTK_MISC (data->iconimage), 0.5, 1);
-  g_object_unref (G_OBJECT (icon));
+
+  if (G_LIKELY (icon))
+    g_object_unref (G_OBJECT (icon));
 
   data->labels = g_array_new (FALSE, TRUE, sizeof (datas));
 
@@ -717,6 +700,7 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
   data->tooltipbox = gtk_event_box_new ();
   gtk_container_add (GTK_CONTAINER (data->tooltipbox), vbox);
   gtk_widget_show_all (data->tooltipbox);
+  GTK_WIDGET_SET_FLAGS (GTK_WIDGET(data->tooltipbox), GTK_NO_WINDOW);
 
   xfce_panel_plugin_add_action_widget (plugin, data->tooltipbox);
 
@@ -746,7 +730,7 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
   gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_cb, data);
+    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
 
   return data;
 }
@@ -757,6 +741,8 @@ static void
 xfceweather_free (XfcePanelPlugin  *plugin,
                   xfceweather_data *data)
 {
+  weather_http_cleanup_qeue ();
+
   if (data->weatherdata)
     xml_weather_free (data->weatherdata);
 
@@ -792,16 +778,6 @@ xfceweather_set_size (XfcePanelPlugin  *panel,
 {
   data->size = size;
 
-  /* arbitrary, choose something that works */
-  if (size <= 30)
-    data->iconsize = GTK_ICON_SIZE_SMALL;
-  else if (size <= 36)
-    data->iconsize = GTK_ICON_SIZE_BUTTON;
-  else if (size <= 48)
-    data->iconsize = GTK_ICON_SIZE_LARGE_TOOLBAR;
-  else
-    data->iconsize = GTK_ICON_SIZE_DND;
-
   gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
 
   if (data->weatherdata)
@@ -809,6 +785,7 @@ xfceweather_set_size (XfcePanelPlugin  *panel,
   else
     set_icon_error (data);
 
+  /* we handled the size */
   return TRUE;
 }
 
@@ -825,6 +802,8 @@ weather_construct (XfcePanelPlugin *plugin)
 
   xfceweather_read_config (plugin, data);
 
+  xfceweather_set_visibility (data);
+
   xfceweather_set_size (plugin, xfce_panel_plugin_get_size (plugin), data);
 
   gtk_container_add (GTK_CONTAINER (plugin), data->tooltipbox);
@@ -842,7 +821,7 @@ weather_construct (XfcePanelPlugin *plugin)
   g_signal_connect (G_OBJECT (plugin), "configure-plugin",
                     G_CALLBACK (xfceweather_create_options), data);
 
-  update_plugin (data, TRUE);
+  update_weatherdata (data);
 }
 
 XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (weather_construct);
